// TestSSL.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include <Windows.h>

#include <openssl/ssl.h>
#include <openssl/rand.h>
#include <openssl/err.h>
#include <openssl/conf.h>

#include "mySSL.h"

#if defined(WIN32)

#define MUTEX_TYPE HANDLE
#define MUTEX_SETUP(x) (x) = CreateMutex(NULL, FALSE, NULL)
#define MUTEX_CLEANUP(x) CloseHandle(x)
#define MUTEX_LOCK(x) WaitForSingleObject((x), INFINITE)
#define MUTEX_UNLOCK(x) ReleaseMutex(x)
#define THREAD_ID GetCurrentThreadId()

#elif defined(_POSIX_THREADS)

/* _POSIX_THREADS is normally defined in unistd.h if pthreads are
available on your platform. */
#define MUTEX_TYPE pthread_mutex_t
#define MUTEX_SETUP(x) pthread_mutex_init(&(x), NULL)
#define MUTEX_CLEANUP(x) pthread_mutex_destroy(&(x))
#define MUTEX_LOCK(x) pthread_mutex_lock(&(x))
#define MUTEX_UNLOCK(x) pthread_mutex_unlock(&(x))
#define THREAD_ID pthread_self()
#else

#error You must define mutex operations appropriate for your platform!

#endif

/* This array will store all of the mutexes available to OpenSSL. */
static MUTEX_TYPE *mutex_buf = NULL;

static void locking_function(int mode, int n, const char * file, int line)
{
	if (mode & CRYPTO_LOCK)
		MUTEX_LOCK(mutex_buf[n]);
	else
		MUTEX_UNLOCK(mutex_buf[n]);
}

static unsigned long id_function(void)
{
	return ((unsigned long)THREAD_ID);
}

int THREAD_setup(void)
{
	int i;

	mutex_buf = (MUTEX_TYPE *)malloc(CRYPTO_num_locks()*sizeof(MUTEX_TYPE));

	if (!mutex_buf)
		return 0;

	for (i = 0; i < CRYPTO_num_locks(); i++)
		MUTEX_SETUP(mutex_buf[i]);

	CRYPTO_set_id_callback(id_function);
	CRYPTO_set_locking_callback(locking_function);

	return 1;
}

int THREAD_cleanup(void)
{
	int i;

	if (!mutex_buf)
		return 0;

	CRYPTO_set_id_callback(NULL);
	CRYPTO_set_locking_callback(NULL);

	for (i = 0; i < CRYPTO_num_locks(); i++)
		MUTEX_CLEANUP(mutex_buf[i]);

	free(mutex_buf);
	mutex_buf = NULL;

	return 1;
}

int init_OpenSSL(void)
{
	if (!THREAD_setup() || !SSL_library_init())
	{
		TRACE(L"** OpenSSL initialization failed!\n");
		return -1;
	}

	SSL_load_error_strings();

	return 0;
}

int close_OpenSSL(void)
{
	if (!THREAD_cleanup())
	{
		TRACE(L"** OpenSSL initialization failed!\n");
		return -1;
	}

	ERR_free_strings();

// 	ENGINE_cleanup(); 
	CONF_modules_unload(0);

	EVP_cleanup();
	CRYPTO_cleanup_all_ex_data();

	return 0;
}

// int _tmain(int argc, _TCHAR* argv[])
// {
// 	int ret;
// 	SSL *pSSL;
// 
// 	init_OpenSSL();
// 
// 	sslInitCTX_c(NULL, NULL, NULL);
// 
// 	ret = sslCreateSSL_c(&pSSL);
// 
// 	close_OpenSSL();
// 
// 	return 0;
// }
// 
