// SSL.cpp : DLL  α׷   մϴ.
//
#include "stdafx.h"

#include "mySSL.h"

// #include <openssl/conf.h>
// #include <openssl/evp.h>
// #include <openssl/engine.h>

#include "sslcommon.h"
#include "map"

#define MAX_LEN 1024*12
#define MARK 0x534D5553

static const int SERVER_SESSION_ID = 1; //anything

static SSL_CTX *pCTX = NULL;

static int password_cb(char *buf, int num, int rwflag, void *userdata)
{
	size_t len;

	len = strlen((const char *)userdata);
	strncpy_s(buf, num, (const char *)userdata, len);

	return len;
}

static DH* tmp_dh_cb(SSL *s, int is_export, int keylength)
{
	//instead of generating keys you can precreate them and load as i have in commented example 
	//please look at dhparam.pod, also SSL_CTX_set_tmp_dh/SSL_CTX_set_tmp_rsa instead of callback
	static std::map<int, DH*> dh_tmp; //RSA* keys are destroyed when app exits
	if(!is_export)
		keylength = 2048; //anything bigger takes a really long time, but might suit you better

	std::map<int, DH*>::iterator it = dh_tmp.find(keylength);

	if(it == dh_tmp.end())
	{
		//can be lengthy operation with large key sizes, but done only 
		//once for a key size during the app's lifetime
		//dh_tmp[keylength]=DH_generate_parameters(keylength, 5, NULL, NULL);

		if(keylength == 512)
			dh_tmp[keylength] = get_dh512();
		else if(keylength == 1024)
			dh_tmp[keylength] = get_dh1024();
		else if(keylength == 2048)
			dh_tmp[keylength] = get_dh2048();
		else
			dh_tmp[keylength] = get_dh4096();
	}

	return dh_tmp[keylength];

	/*BIO *bio;
	if ((bio=BIO_new_file("..\\..\\dh1024.pem","r")) == NULL)
	int i3=0;

	DH *ret=PEM_read_bio_DHparams(bio,NULL,NULL,NULL);
	BIO_free(bio);*/
}

static RSA* tmp_rsa_cb(SSL *s, int is_export, int keylength)
{
	//not tested for rsa, but should work just as above for dh
	static std::map<int, RSA*> rsa_tmp; //RSA* keys are destroyed when app exits
	if(!is_export)
		keylength = 2048;

	std::map<int, RSA*>::iterator it = rsa_tmp.find(keylength);
	if(it == rsa_tmp.end())
		rsa_tmp[keylength] = RSA_generate_key(keylength,RSA_F4,NULL,NULL);

	return rsa_tmp[keylength];
}

int sslInitCTX_c(char *CAlist, char *keyfile, char *passwd)
{
	if (pCTX != NULL)
		return 0;

	if (RAND_status() != 1)
		return -1;

	//SSL Context 
	SSL_METHOD *pMethod;
	pMethod = TLSv1_client_method();
	pCTX = SSL_CTX_new(pMethod);
	if (pCTX == NULL)
		return -1;

	SSL_CTX_set_cipher_list(pCTX, "HIGH:MEDIUM:@STRENGTH");

	if (CAlist == NULL || 
		keyfile == NULL ||
		passwd == NULL)
	{
		SSL_CTX_set_verify(pCTX, SSL_VERIFY_NONE, NULL);
		return 0;
	}

	if(!SSL_CTX_load_verify_locations(pCTX, CAlist, 0) || 
		!SSL_CTX_set_default_verify_paths(pCTX))
		return -1;

	// Ű  
	SSL_CTX_set_default_passwd_cb(pCTX, password_cb);
	SSL_CTX_set_default_passwd_cb_userdata(pCTX, passwd);

	if (!(SSL_CTX_use_certificate_chain_file(pCTX, keyfile)))
		return -1;
	
	if (!(SSL_CTX_use_PrivateKey_file(pCTX, keyfile, SSL_FILETYPE_PEM)))
		return -1;

	// ŷϴ CA 
	if (!(SSL_CTX_load_verify_locations(pCTX, CAlist, NULL)))
		return -1;

	return 1;
}

int sslFreeCTX()
{
// 	ERR_free_strings();

//	ERR_remove_state(0);
//
//// 	FILE *fout;
//// 	fout = fopen("d:\err.txt", "w"); 
//// 	CRYPTO_mem_leaks_fp(fout);
//// 	fclose(fout);
//
	if (pCTX != NULL)
	{
		SSL_CTX_free(pCTX);
		pCTX = NULL;
	}
//
//	CRYPTO_set_id_callback(NULL);
//	CRYPTO_set_locking_callback(NULL);
//
////  	ENGINE_cleanup(); 
//// 	CONF_modules_unload(0);
//
//	EVP_cleanup();
//	CRYPTO_cleanup_all_ex_data();
//	
	return 1;
}

int sslCreateSSL_c(SSL **pSSL)
{
	BIO *rbio;
	BIO *wbio;
	
	*pSSL = SSL_new(pCTX);
	if (*pSSL == NULL)
		return -1;

	//BIO allocation for write
	rbio = BIO_new(BIO_s_mem()); 
	if (rbio == NULL)
		return -1;
	
	//BIO allocation for read
	wbio = BIO_new(BIO_s_mem()); 
	if (wbio == NULL)
		return -1;

	SSL_set_bio(*pSSL, rbio, wbio);	//can't fail
	BIO_set_close(rbio, BIO_CLOSE);	//always returns 1
	BIO_set_close(wbio, BIO_CLOSE);	//always returns 1

	//SSL Ʈ Ŭ̾Ʈ忡 ۾ϵ Ѵ.
	SSL_set_connect_state(*pSSL);

	return 1;
}

int sslCheckCert(SSL *pSSL, char *host)
{
	int err;
	X509 *pPeerCert;
	char peer_CN[256];

	err = SSL_get_verify_result(pSSL);
	if (err != X509_V_OK)
		return err;
	/*Check the cert chain. The chain length
	is automatically checked by OpenSSL when
	we set the verify depth in the ctx */

	/*Check the common name*/
	pPeerCert = SSL_get_peer_certificate(pSSL);
	X509_NAME_get_text_by_NID(X509_get_subject_name(pPeerCert),
							NID_commonName, peer_CN, 256);
	
	return strcmp(peer_CN, host);
}

int sslHandShake_c(SSL *pSSL, BYTE *pRespond, int len)
{
	int ret;
	size_t nRead;
	
	nRead = BIO_ctrl_pending(SSL_get_wbio(pSSL));
	if (nRead > 0)
		return -1;

	if (pRespond != NULL && len != 0)
		BIO_write(SSL_get_rbio(pSSL), pRespond, len);

	ret = SSL_connect(pSSL); //TLS/SSL handshake

	if(ret <= 0)
	{
		ret = SSL_get_error(pSSL, ret);
		if (ret == SSL_ERROR_SSL)
			return -1;
	}

	return ret;
}

